from datetime import datetime, timedelta
import functools


def timed_cache(**timedelta_kwargs):

    def _wrapper(f):
        update_delta = timedelta(**timedelta_kwargs)
        next_update = datetime.utcnow() - update_delta
        # Apply @lru_cache to f with no cache size limit
        f = functools.lru_cache(None)(f)

        @functools.wraps(f)
        def _wrapped(*args, **kwargs):
            nonlocal next_update
            now = datetime.utcnow()
            if now >= next_update:
                f.cache_clear()
                next_update = now + update_delta
            return f(*args, **kwargs)
        return _wrapped
    return _wrapper



from sqlparse import parse
from sqlparse.tokens import Keyword, DML
from sqlparse.sql import Identifier, Where


__all__ = [
  'is_dml',
]


def is_dml(sql):
    parsed_list = parse(sql)
    for parsed in parsed_list:
        if parsed.token_first(skip_cm=True).ttype is not DML:
            return False
    return True


def update_or_delete_to_select(statement):
    return 'SELECT * FROM ' + \
           str(list(filter(
               lambda x: isinstance(x, Identifier),
               statement))[0])\
           + ' ' + \
           str(list(filter(
               lambda x: isinstance(x, Where),
               statement))[-1])


def find_tables(sql):
    parsed_list = parse(sql)
    tables = set()
    for parsed in parsed_list:
        found_key = -1
        for index, item in enumerate(parsed):
            if item.value.upper() in ['INTO', 'FROM', 'UPDATE']:
                found_key = index
            elif found_key != -1:
                if item.ttype is Identifier or item.ttype is None:
                    tables.add(item.get_name())
                elif item.ttype is Keyword:
                    found_key = -1
    return list(filter(None, tables))


# Input a sql
# Return a select only SQL list
def selectify(sentence: str):
    for statement in parse(sentence):
        if statement.get_type() in ['UPDATE', 'DELETE']:
            yield update_or_delete_to_select(statement)


a = selectify('update eee where id = 4; update aaa where id =4')


if __name__ == '__main__':
    # print(bar.has_ancestor(Keyword))
    sql = '1; select; select 1; select * from 2'
    foo = is_dml(sql)
    print(foo)
    # print(type(foo))
    # print(dir(foo))
    # print(foo[0].value)
    # print(type(foo[0]))
    # print(type(foo.get_type()))



from urllib.parse import quote, urljoin, urlencode
from itertools import chain


def merge_fields(request):
    # Return a mixture of all provided objects in request
    data = {}
    data.update(request.values.to_dict())
    data.update(request.get_json(force=True, silent=True) or {})
    return data


def create_url(base, *query, path=None, params={}):
    """ Create a url.

    Creates a url by combining base, path, and the query's list of
    key/value pairs. Escaping is handled automatically. Any
    key/value pair with a value that is None is ignored.

    Keyword arguments:
    base -- The left most part of the url (ex. http://localhost:5000).
    path -- The path after the base (ex. /foo/bar).
    query -- A list of key value pairs (ex. [('key', 'value')]).

    Example usage:
    >>> create_url(
    ...     'http://localhost:5000',
    ...     ('key1', 'value'),
    ...     ('key2', None),     # Will not include None
    ...     ('url', 'http://example.com'),
    ...
    ...     path='foo/bar',
    ...     key=['value1', 'value2']
    ...
    ... )
    'http://localhost:5000/foo/bar?key1=value&url=http%3A%2F%2Fexample.com&key=value1&key=value2'
    """
    url = base
    # Add the path to the url if it's not None.
    if path is not None:
        url = urljoin(url, quote(path))
    # Remove key/value pairs with None values.
    query = chain(
        filter(lambda pair: pair[1] is not None, query), params.items())

    def seperate(entry):
        if isinstance(entry[1], list):
            return ((entry[0], i) for i in entry[1] if i)
        elif entry[1] is None:
            return ()
        return (entry,)
    query = list(chain(*map(seperate, query)))
    # Add the query string to the url
    url = urljoin(url, '?{0}'.format(urlencode(list(query))))
    return url
